#include <ros/ros.h>

#include <cstring>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

#include "vl53l0x_def.h"
#include "vl53l0x_i2c_comms.h"

void VL53L0X_i2c_init(const char *path, VL53L0X_DEV dev) {
  int i2c_dev_file = open(path, O_RDWR);
  if (i2c_dev_file < 0) {
    ROS_FATAL("Couldn't open %s: %s", path, strerror(errno));
    return;
  }

  if (ioctl(i2c_dev_file, I2C_SLAVE, VL53L0X_I2C_ADDR) < 0) {
    close(i2c_dev_file);
    ROS_FATAL("Couldn't set slave address: %s", strerror(errno));
    return;
  }

  ROS_INFO("Successfully opened I2C device & set address");
  dev->fd = i2c_dev_file;
}

extern "C" {
#include "vl53l0x_i2c_platform.h"
#include "vl53l0x_platform.h"

int VL53L0X_write_multi(uint8_t address, uint8_t index, uint8_t *pdata,
                        int32_t count, int fd) {
  uint8_t *buf = (uint8_t *)malloc(count + 1);
  buf[0] = index;
  memcpy(&buf[1], pdata, count);
  int ret = write(fd, buf, count + 1);
  if (ret == count + 1) {
    return VL53L0X_ERROR_NONE;
  } else if (ret < 0) {
    ROS_ERROR("Error while writing to I2C device: %s", strerror(errno));
  } else {
    ROS_ERROR("Only wrote %d bytes (of %d requested)", ret, count + 1);
  }
  return VL53L0X_ERROR_CONTROL_INTERFACE;
}

int VL53L0X_read_multi(uint8_t address, uint8_t index, uint8_t *pdata,
                       int32_t count, int fd) {
  int ret = VL53L0X_write_multi(address, index, NULL, 0, fd);
  if (ret != VL53L0X_ERROR_NONE) {
    return ret;
  }

  ret = read(fd, pdata, count);
  if (ret == count) {
    return VL53L0X_ERROR_NONE;
  } else if (ret < 0) {
    ROS_ERROR("Error while reading from I2C device: %s", strerror(errno));
  } else {
    ROS_ERROR("Only read %d bytes (of %d requested)", ret, count);
  }
  return VL53L0X_ERROR_CONTROL_INTERFACE;
}

int VL53L0X_write_byte(uint8_t deviceAddress, uint8_t index, uint8_t data,
                       int fd) {
  return VL53L0X_write_multi(deviceAddress, index, &data, 1, fd);
}

int VL53L0X_write_word(uint8_t deviceAddress, uint8_t index, uint16_t data,
                       int fd) {
  uint8_t buff[2];
  buff[1] = data & 0xFF;
  buff[0] = data >> 8;
  return VL53L0X_write_multi(deviceAddress, index, buff, 2, fd);
}

int VL53L0X_write_dword(uint8_t deviceAddress, uint8_t index, uint32_t data,
                        int fd) {
  uint8_t buff[4];

  buff[3] = data & 0xFF;
  buff[2] = data >> 8;
  buff[1] = data >> 16;
  buff[0] = data >> 24;

  return VL53L0X_write_multi(deviceAddress, index, buff, 4, fd);
}

int VL53L0X_read_byte(uint8_t deviceAddress, uint8_t index, uint8_t *data,
                      int fd) {
  return VL53L0X_read_multi(deviceAddress, index, data, 1, fd);
}

int VL53L0X_read_word(uint8_t deviceAddress, uint8_t index, uint16_t *data,
                      int fd) {
  uint8_t buff[2];
  int r = VL53L0X_read_multi(deviceAddress, index, buff, 2, fd);

  uint16_t tmp;
  tmp = buff[0];
  tmp <<= 8;
  tmp |= buff[1];
  *data = tmp;

  return r;
}

int VL53L0X_read_dword(uint8_t deviceAddress, uint8_t index, uint32_t *data,
                       int fd) {
  uint8_t buff[4];
  int r = VL53L0X_read_multi(deviceAddress, index, buff, 4, fd);

  uint32_t tmp;
  tmp = buff[0];
  tmp <<= 8;
  tmp |= buff[1];
  tmp <<= 8;
  tmp |= buff[2];
  tmp <<= 8;
  tmp |= buff[3];

  *data = tmp;

  return r;
}
} // extern "C"
